﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using GeneratorLib;
using System.IO;

namespace Zobrazovani
{
    public partial class Form1 : Form
    {
        public Vez Vez;
        public int YVeze;
        public int XVeze;
        Orientace AktualniOrientace = Orientace.Zepredu;

        public Form1()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                var lines = File.ReadAllLines(openFileDialog1.FileName);
                Vez = new Vez(lines);

                var splat = lines[0].Split(' ').Where(l=>!string.IsNullOrWhiteSpace(l)).ToArray();

                XVeze = int.Parse(splat[0]);
                YVeze = int.Parse(splat[1]);
            }

            Reload();
        }

        private void button2_Click(object sender, EventArgs e)
        {
            AktualniOrientace = Orientace.Zepredu;
            Reload();
        }

        private void button3_Click(object sender, EventArgs e)
        {
            AktualniOrientace = Orientace.Zprava;
            Reload();
        }

        private void button4_Click(object sender, EventArgs e)
        {
            AktualniOrientace = Orientace.Zleva;
            Reload();
        }

        private void button5_Click(object sender, EventArgs e)
        {
            AktualniOrientace = Orientace.Zepredu;
            Reload();
        }

        private void button6_Click(object sender, EventArgs e)
        {
            AktualniOrientace = Orientace.Shora;
            Reload();
        }

        private void button7_Click(object sender, EventArgs e)
        {
            AktualniOrientace = Orientace.Zdola;
            Reload();
        }


        public void Reload()
        {
            if (Vez == null) return;

            Graphics g = pictureBox1.CreateGraphics();

            g.Clear(Color.White);

            var height = pictureBox1.Size.Height * 0.9f;
            var width = pictureBox1.Size.Width * 0.9f;


            var pen = new Pen(Color.Black);

            var prilehle = Vez.Kostky.Where(JePrilehla).ToArray();

            foreach (var kostka in prilehle)
            {
                Color c;

                switch (kostka.Barva)
                {
                    case "A": c = Color.FromArgb(0, 255, 255); break;
                    case "R": c = Color.FromArgb(255, 0, 0); break;
                    case "G": c = Color.FromArgb(0, 255, 0); break;
                    case "B": c = Color.FromArgb(0, 0, 255); break;
                    case "Y": c = Color.FromArgb(255, 255, 0); break;
                    default: throw new IndexOutOfRangeException();
                }

                Rectangle rect = GetRect(kostka);
                g.FillRectangle(new Pen(c).Brush, rect);
                g.DrawRectangle(new Pen(Color.Black), rect);
            }

        }

        public bool JePrilehla(UmisteniKostky kostka)
        {
            switch (AktualniOrientace)
            {
                case Orientace.Zepredu:
                    return KostkaZasahujeNaPozici(kostka, -100, 1);
                case Orientace.Zezadu:
                    return KostkaZasahujeNaPozici(kostka, -100, YVeze);
                case Orientace.Zleva:
                    return KostkaZasahujeNaPozici(kostka, 1, -100);
                case Orientace.Zprava:
                    return KostkaZasahujeNaPozici(kostka, XVeze, -100);
                case Orientace.Shora:
                    return kostka.Patro == Vez.Vyska;
                case Orientace.Zdola:
                    return kostka.Patro == 1;
                default:
                    break;
            }

            return true;
        }

        public bool KostkaZasahujeNaPozici(UmisteniKostky kostka, int px, int py)
        {
            for (int x = kostka.X; x < kostka.X + kostka.VelikostX; x++)
            {
                for (int y = kostka.Y; y < kostka.Y + kostka.VelikostY; y++)
                {
                    if (x == px || y == py) return true;
                }
            }

            return false;
        }

        public Rectangle GetRect(UmisteniKostky kostka)
        {
            Rectangle rect = new Rectangle();

            if (AktualniOrientace == Orientace.Zepredu || AktualniOrientace == Orientace.Zezadu || AktualniOrientace == Orientace.Zleva || AktualniOrientace == Orientace.Zprava)
            {

                rect.Height = (int)(pictureBox1.Size.Height * 0.9f / Vez.Vyska);
                rect.Y = (int)(pictureBox1.Height * 0.9 - ((int)((kostka.Patro - 1) * pictureBox1.Size.Height * 0.9f / Vez.Vyska) + 10));

                if (AktualniOrientace == Orientace.Zepredu || AktualniOrientace == Orientace.Zezadu)
                {
                    rect.Width = (int)(pictureBox1.Size.Width * 0.9f / XVeze * kostka.VelikostX);
                    rect.X = (int)(pictureBox1.Size.Width * 0.9f / XVeze * (kostka.X - 1)) + 10;

                    if (AktualniOrientace == Orientace.Zezadu) rect.X = (int)(pictureBox1.Width * 0.9 - rect.X);
                }
                else
                {
                    rect.Width = (int)(pictureBox1.Size.Width * 0.9f / YVeze * kostka.VelikostY);
                    rect.X = (int)(pictureBox1.Size.Width * 0.9f / YVeze * (kostka.Y - 1)) + 10;

                    if (AktualniOrientace == Orientace.Zleva) rect.X = (int)(pictureBox1.Width * 0.9 - rect.X);
                }
            }
            else
            {
                rect.Width = (int)(pictureBox1.Size.Width * 0.9f / XVeze * kostka.VelikostX);
                rect.X = (int)(pictureBox1.Size.Width * 0.9f / XVeze * kostka.X);

                rect.Height = (int)(pictureBox1.Size.Height * 0.9f / YVeze * kostka.VelikostY);
                rect.Y = (int)(pictureBox1.Size.Height * 0.9f / YVeze * kostka.Y);

            }

            return rect;
        }

        private void button8_Click(object sender, EventArgs e)
        {
            Reload();
        }
    }

    enum Orientace
    {
        Zepredu, Zezadu, Zleva, Zprava, Shora, Zdola
    }
}
